/****************************************************************
**
** Implementation of CIGLWIdget class
**
****************************************************************/

#include <GL/glew.h>
#include "CIGLWidget.h"
#include "MMSG.h"

#include <QMessageBox.h>
#include <QImage.h>
#include <QdateTime.h>

using namespace vcg;

CIGLWidget::CIGLWidget( QWidget *parent, const char *name ): QGLWidget(parent, name)
{sg=0;
}
	
void CIGLWidget::initializeGL()
{   
	editing=false;
	GLenum err = glewInit();
	if (GLEW_VERSION_1_3)
	{
		//QMessageBox::information(this,"GLversion","Opengl1.3 supported!");
	}


  // Set up the rendering context, define display lists etc.:
  glClearColor( 0.0, 1.0, 1.0, 0.0 );
	float spec[4]={.4f,.4f,.4f,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);
	
	// Se scommentato disegna solo in wireframe
	//glPolygonMode(GL_FRONT,GL_LINE);
	tb.Init(size().width(),size().height());
	rot.SetIdentity();

	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	if(sg) sg->root.initializeGL();
}

void CIGLWidget::resizeGL( int w, int h )
{
  // setup viewport, projection etc.:
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(60,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	tb.Resize( w,h );

}

void CIGLWidget::setEditing(bool val)
{
	editing=val;
}

void CIGLWidget::paintGL()
{
	glMatrixMode (GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT  );
	glLoadIdentity();
	gluLookAt(2,5,6,0,0,0,0,1,0);
	glMultMatrix(rot);
	glColor3f(1.0f,.4f,.4f);
	static int cnt=0;
	QTime tm;
	if(sg) 	sg->drawGL(tm.elapsed()/1000.0f);
}

void CIGLWidget::mousePressEvent(QMouseEvent*e)
{
  if(!editing) {
		tb.MouseDown( e->x(), e->y(), 0 );
		update();
	}
}

void CIGLWidget::mouseMoveEvent(QMouseEvent*e)
{
	if(!editing) {
		tb.CalcRotMatrix( rot, e->x(), e->y());
		update();
	}
}

void CIGLWidget::mouseReleaseEvent(QMouseEvent*e)
{
  if(!editing) {
		tb.MouseUp( e->x(), e->y());
		update();
	}
}
